# caenamp.tcl---CAEN amplifier control frame


set frame CAEN
set frametitle "CAEN Amplifier Setup & Monitor"
set icontitle "CAEN"

set started 0

if {[info exists env(CAENetserver)]} then {set resource $env(CAENetserver)} \
    else {set resource CAEN}

set mname CAENet.V288
#set V288Address 0x500000
set V288Address  0xf00000

set savedirectory CAENAmpSetup

set amplifier 1
set channel 0
set allamps 0
set allchans 0

set selectampmenu {1 2 3 4 5 6 7 8 9 10}
set selectchanmenu   {0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15}
set amplist $selectampmenu
set chanlist $selectchanmenu

set Status ""

set widgets "FGAIN CGAIN PZ SHAPE POLARITY CONFIG"
set params  "fg cg pz s p c"


#    this is to allow the command just issued to be executed by the amp module
proc short_delay {} {after 10}
#    a longer delay needed when setting all channels
proc long_delay {} {after 100}


proc define-CAENetV288-registers {} {
   global resource mname

   set register "/$resource/$mname.*"
   set-footer "deleting register $register"
   EG "register :pattern '$register' !delete"
   clear-footer

   define-register Address
   define-register Execute
   define-register Reset
   define-register Status
   define-register Data
}

proc define-register {name} {
   global resource mname

   set register "/$resource/$mname.$name"
   set attributes "-C $name"
   set-footer "defining register $register"
   EG "register :name '$register' :class 'xV288' !new"
   EG "register :name '$register' :object '$attributes'"
   clear-footer
}

proc write-register {name value} {
   global resource mname
   set register "/$resource/$mname.$name"
#   set-footer "writing $value to $register"
   EG "register :name '$register' :value '$value'"
#   clear-footer
}

proc read-register {name} {
   global resource mname
   set register "/$resource/$mname.$name"
#   set-footer "reading $register"
   set value [EG "register :name '$register' ?value"]
#   clear-footer
   return $value
}

proc click-amplifier v {
   global amplifier
   set amplifier $v
   redisplay
   read_ident
}

proc click-channel v {
   global channel
   set channel $v
   redisplay
}

proc click-select v {
    global setting
    click-channel $v
    set_param fg $setting(0)
    redisplay
}

proc repaint {} {
   global allamps allchans
   global channel amplifier
  :modify AMP-NUM    :value $amplifier
  :modify ALL-AMPS   :value $allamps
  :modify CHAN-NUM   :value $channel
  :modify ALL-CHANs  :value $allchans
}

proc click-save {w} {
   global env Status savedirectory
   global amplist chanlist amplifier
   global setting

   catch {file delete [file join $env(EXPERIMENTDIR) $savedirectory]}
   if {![file exists [file join $env(EXPERIMENTDIR) $savedirectory]]} then {
      set z [catch {:file mkdir [file join $env(EXPERIMENTDIR) $savedirectory]} m]
      if {$z != 0} then {midas-warning "Cannot create directory [file join $env(EXPERIMENTDIR) $savedirectory]\n $m"}
   }

   set currentamp $amplifier

   if {$w == "ALL"} then {
      set amps $amplist
   } else {
      set amps $amplifier
   }

   set-busy
   set contents ""
   set r 0
   foreach amplifier $amps {
         set Status {}
         set-footer "Saving settings for amplifier $amplifier..."
         append contents "Settings saved for amplifier $amplifier"

#    obtain values for this amplifier
 
         foreach chan $chanlist {
             set r [read_ch $chan]
             if {$r != 0} then {break}
             lappend Status [list $setting(0) $setting(1) $setting(2) $setting(3) $setting(4) $setting(5)]
         }
         if {$r != 0} then {break}
         lappend Status $setting(6)

#    save amplifier status to file
         set f [open [file join $env(EXPERIMENTDIR) $savedirectory amp$amplifier] w]
         puts $f $Status
         close $f
         append contents " to file [file join $env(EXPERIMENTDIR) $savedirectory amp$amplifier]\n"
   }
   clear-busy

   set amplifier $currentamp

   if {$r != 0} then {
      midas-warning "Save of amplifier settings to $env(EXPERIMENTDIR)/$savedirectory abandoned"
      clear-footer
      redisplay
      return
   }

   if {$contents == ""} then {
      midas-warning "Warning - Nothing has been saved!"
   } else {
      midas-report "$contents"
   }

   set-footer "Amplifier settings saved to $env(EXPERIMENTDIR)/$savedirectory"

   redisplay
}

proc click-restore {w} {
   global env Status savedirectory
   global amplist chanlist amplifier channel allchans
   global params

   set currentamp $amplifier
   set currentchan $channel
   set allchans 0

   if {$w == "ALL"} then {
      set amps $amplist
   } else {
      set amps $amplifier
   }

   set-busy
   set contents ""
   set r 0
   foreach amplifier $amps {
         set-footer "Restoring settings for amplifier $amplifier..."
         append contents "Settings restored for amplifier $amplifier"

#    read amplifier settings from file
         set f [open [file join $env(EXPERIMENTDIR) $savedirectory amp$amplifier] r]
         set Status [gets $f]
         close $f
         append contents " from file [file join $env(EXPERIMENTDIR) $savedirectory amp$amplifier]\n"

#    restore values for this amplifier

         set c 0
         foreach channel $chanlist {
             set settings [lindex $Status $c]
             incr c
             set p 0
             foreach param $params {
                 set r [set_param $param [lindex $settings $p]]
                 incr p
                 if {$r != 0} then {break}
             }
             if {$r != 0} then {break}
         }
         if {$r != 0} then {break}

         set r [set_offset [last $Status]]
         if {$r != 0} then {break}
   }
   clear-busy

   set amplifier $currentamp
   set channel $currentchan

   if {$r != 0} then {
      midas-warning "Restore of amplifier settings from $env(EXPERIMENTDIR)/$savedirectory abandoned"
      clear-footer
      redisplay
      return
   }

   if {$contents == ""} then {
      midas-warning "Warning - Nothing has been restored!"
   } else {
      midas-report "$contents"
   }

   set-footer "Amplifier settings restored from $env(EXPERIMENTDIR)/$savedirectory"

   redisplay
}


proc do-update {} {
   global channel
   read_ch $channel
}

proc redisplay {} {repaint; do-update}

proc click arg {
   global debug allamps allchans
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit; return}
   switch $w {
      AMP-SEL     -
      AMP-NUM     {click-amplifier $v}
      AMPSEL      {click-crateselection $v}
      ALL-AMPS    {set allamps $v}

      CHAN-SEL      -
      CHAN-NUM      {click-channel $v}
      CHANSEL       {click-ampselection $v}
      ALL-CHANS     {set allchans $v}

      PROBE        {probe_for_amplifiers}

      FGAIN      {set_param fg  $v; redisplay}
      CGAIN      {set_param cg  $v; redisplay}
      PZ         {set_param pz  $v; redisplay}
      SHAPE      {set_param s  $v; redisplay}
      POLARITY   {set_param p  $v; redisplay}
      CONFIG     {set_param c  $v; redisplay}
      OFFSET     {set_offset $v}

      SELECT     {click-select $v}
      MPX        {set_mpx $v; redisplay}

      SAVE         {click-save CURRENT}
      SAVEALL      {click-save ALL}
      RESTORE      {click-restore CURRENT}
      RESTOREALL   {click-restore ALL}

      REDISPLAY    {redisplay}
      default      {puts stdout $arg}
   }
}


proc make-frame {} {
  global env
  global selectampmenu selectchanmenu

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
     :layout H; :row-gap 10

      :number AMP-NUM  :label "Amplifier number:" :min 1 :max 8 :w 5  :valuex 20c
      :menubutton AMP-SEL :label "select:" :strings $selectampmenu
      :hskip 10
      :choice ALL-AMPS :label "Act on Amplifiers?" :valuex 18c :strings [list Current All] :value 0
      :next-row T
      :number CHAN-NUM  :label "Channel number:" :min 0 :max 15 :w 5 :valuex 20c
      :menubutton CHAN-SEL :label "select:" :strings $selectchanmenu
      :hskip 10
      :choice ALL-CHANS :label "Act on Channels?" :valuex 18c :strings [list Current All] :value 0
#      :next-row T
#      :message M1 :label "Amplifiers in use" :bold T
#      :hskip 30
#      :choice AMPSINUSE :label "" :exclusive F :strings $selectampmenu :value 0 :deaf T

    :panel P2 :layout R :padx 10
      :choice SELECT  :label MUX :exclusive T :strings $selectchanmenu :w 3 :layout V

    :panel P1 :layout T :padx 10
      :layout V; :col-gap 15; :row-gap 15

      :slider FGAIN     :label "Fine Gain"   :min 0 :max 255  :valuex 30c
      :slider CGAIN     :label "Course Gain" :min 0 :max 7    :valuex 30c
      :slider PZ        :label "Pole Zero"   :min 0 :max 255  :valuex 30c
      :choice SHAPE     :label "Shaping time (us)"  :exclusive T :strings [list 0.2 1 3 6] :w 4 :valuex 30c
      :choice POLARITY  :label "Output Polarity"      :exclusive T :strings [list Positive Negative] :valuex 30c
      :choice CONFIG    :label "Output Configuration" :exclusive T :strings [list Direct Inverted]  :valuex 30c
      :slider OFFSET    :label "Offset"   :min 0 :max 255  :valuex 30c

    :panel CLICK :layout B :padx 10 :pady 10 
      :layout H; :col-gap 25; :row-gap 5
      :button SAVEALL     :label "Save All amplifiers"
      :button SAVE        :label "Save Current amplifier"
      :next-row T
      :button RESTOREALL  :label "Restore All amplifiers"
      :button RESTORE     :label "Restore Current amplifier"
      :next-row T
      :button REDISPLAY
      :checkbox MPX     :label "Multiplex Output"
      :button PROBE       :label "Probe for amplifiers"
      :button HELP  :command click-help

      :show T

}


proc read_ch {n} {
   global widgets setting
   global amplifier 
   global response

    catch {unset setting}

    set OpCode [expr $n << 8 | 3]

#    write command  to the Tx buffer

    write-register Data "$amplifier $OpCode"

#    execute the CAENet operation

    short_delay

    set status [read-register Execute]
    if {$status != 0 && $status != 7} then {
        set status [expr $status & 255]
        if {$status == 255} then {
            midas-warning "The amplifier is not responding\nCheck that amplifier $amplifier does exist and that the NIM bin is switched on"
        } else {
            midas-warning "CAENet: Error number $status received"
        }
        return 1
    }

#    read the received data

    set response [read-register Data]

  set setting(0)  [lindex $response 0]
  set setting(1)  [expr [lindex $response 2] & 7]
  set setting(2)  [lindex $response 1]
  set setting(3)  [expr [lindex $response 2] >> 3 & 3]
  set setting(4)  [expr [lindex $response 2] >> 6 & 1]
  set setting(5)  [expr [lindex $response 2] >> 5 & 1]

  for {set i 0} {$i < 6} {incr i} {
     set widget [lindex $widgets $i]
     :modify $widget :value $setting($i)
  }

#    finally read OFFSET

    set OpCode 2

#    write command  to the Tx buffer

    write-register Data "$amplifier $OpCode"

    short_delay

#    execute the CAENet operation

    set status [read-register Execute]
    if {$status != 0 && $status != 7} then {
        set status [expr $status & 255]
        if {$status == 255} then {
            midas-warning "The amplifier is not responding\nCheck that amplifier $amplifier does exist and that the NIM bin is switched on"
        } else {
            midas-warning "CAENet: Error number $status received"
        }
        return 1
    }

#    read the received data

    set response [read-register Data]

    set setting(6) [lindex $response 0]

   :modify OFFSET :value [lindex $response 0]

   read_mpx

   return 0
}


proc read_ident {} {
   global V288Address
   global amplifier 
   global response

   read-register Reset

#    write OpCode 0  -  module identifier  -  to the Tx buffer

    write-register Data "$amplifier 0"

#    execute the CAENet operation

     short_delay

    set status [read-register Execute]
    if {$status != 0 && $status != 7} then {
        set status [expr $status & 255]
        if {$status == 255} then {
            midas-warning "The amplifier is not responding\nCheck that amplifier $amplifier does exist and that the NIM bin is switched on"
        } else {
            midas-warning "CAENet: Error number $status received"
        }
        return
    }

#    read the received data

    set response [read-register Data]

    set n418 ""
    for {set i 0} {$i < 16} {incr i} {
       append n418 [format %c [lindex $response $i]]
    }

     set-footer "The amplifier has answered: $n418"
}

proc probe_for_amplifiers {} {
   global selectampmenu amplist amplifier 

   set amplist {}
   foreach amp $selectampmenu {

       set-footer "Probing for amplifier $amp"

#    write OpCode 0  -  module identifier  -  to the Tx buffer

       write-register Data "$amp 0"

    short_delay

#    execute the CAENet operation

       set status [read-register Execute]

       if {$status == 0 || $status == 7} then {lappend amplist $amp}
   }
   
   clear-footer

   :modify AMP-SEL :strings $amplist
    
   if {[llength $amplist] == 0} then {
       midas-warning "No amplifiers where detected\nCheck connections and NIM bin power"
   } else {
       set amplifier [first $amplist]
   }
}

        
proc set_param {p v} {  
   global channel allchans allamps
   global amplist chanlist amplifier 
   global response

#    note ch is channel number (0-15) or all channels (16)

  switch $p {
     fg     {set opcode 0x10}
     cg     {set opcode 0x11}
     pz     {set opcode 0x12}
     s      {set opcode 0x13}
     p      {set opcode 0x14}
     c      {set opcode 0x15}
     default   {return}
  }                
 
#    We do not use the inbuilt set-all because it always returns an error 
#    although it does it

#   if {$allchans == 0} then {set ch $channel} else {set ch 16}
#   set OpCode [expr $ch << 8 | $opcode]

   if {$allchans == 0} then {set chs $channel} else {set chs $chanlist}

   if {$allamps == 0} then {set amps $amplifier} else {set amps $amplist}
   set currentamp $amplifier

   foreach amplifier $amps {
     foreach ch $chs {
 
        set OpCode [expr $ch << 8 | $opcode]

#    write command  to the Tx buffer

       write-register Data "$amplifier $OpCode $v"

       short_delay

#    execute the CAENet operation

       set status [read-register Execute]
       if {$status != 0} then {
          set status [expr $status & 255]
          if {$status == 255} then {
            midas-warning "The amplifier is not responding\nCheck that amplifier $amplifier does exist and that the NIM bin is switched on"
          } else {
            midas-warning "CAENet: Error number $status received"
          }
          set amplifier $currentamp
          redisplay
          return 1
       }
   } }

   set amplifier $currentamp
   return 0
}

proc set_offset {v} {
   global amplifier allamps amplist
   global response

    set OpCode 0x16

   if {$allamps == 0} then {set amps $amplifier} else {set amps $amplist}
   set currentamp $amplifier

   foreach amplifier $amps {

#    write command  to the Tx buffer

       write-register Data "$amplifier $OpCode $v"

       short_delay

#    execute the CAENet operation

       set status [read-register Execute]
       if {$status != 0} then {
          set status [expr $status & 255]
          if {$status == 255} then {
            midas-warning "The amplifier is not responding\nCheck that amplifier $amplifier does exist and that the NIM bin is switched on"
          } else {
            midas-warning "CAENet: Error number $status received"
          }
          set amplifier $currentamp
          redisplay
          return 1
       }
   }

   set amplifier $currentamp
   redisplay
   return 0
}

proc init {} {
   global EGreport V288Address

   set EGreport -1

   define-CAENetV288-registers

   write-register Address $V288Address
}

proc start args {
   global started

   if {$started == 0} then {make-frame; set started 1} else {open-frame}

   init
   if {$started < 2} then {probe_for_amplifiers; set started 2}
   read_ident

   redisplay
}


#    procedures added to enable and disable the MPX outputs
#    note that these are new and may not be in older modules

proc set_mpx {vl} {
   global amplifier 

#    write OpCode 0x20 (disable) or 0x21 (enable)  -  to the Tx buffer

    write-register Data "$amplifier [expr 0x20 + $vl]"

#    execute the CAENet operation

     short_delay

    set status [read-register Execute]
    if {$status != 0 && $status != 7} then {
        set status [expr $status & 255]
        if {$status == 255} then {
            midas-warning "The amplifier is not responding\nCheck that amplifier $amplifier does exist and that the NIM bin is switched on"
        } else {
            midas-warning "CAENet: Error number $status received"
        }
        return
    }
}

proc read_mpx {} {
   global amplifier 
   global response

    set OpCode 4

#    write OpCode 4  - read status  -  to the Tx buffer

    write-register Data "$amplifier $OpCode"

#    execute the CAENet operation

     short_delay

    set status [read-register Execute]
    if {$status != 0 && $status != 7} then {
        set status [expr $status & 255]
        if {$status == 255} then {
            midas-warning "The amplifier is not responding\nCheck that amplifier $amplifier does exist and that the NIM bin is switched on"
        } else {
            midas-warning "CAENet: Error number $status received"
        }
        return
    }

#    read the received data

    set response [read-register Data]
    :modify MPX :value [expr $response >> 7]
    :modify SELECT :value [expr $response & 15]
}
