#    f2vbdiag.tcl  -  F2VB module diagnostic frame

set frame f2vbdiag
set frametitle "F2VB Diagnostics Frame"
set icontitle "F2VBdiag"


set mname "F2VB"
set DataWidth 16

if {[info exists env(F2VBserver)]} then {set resource $env(F2VBserver)} else {set resource vme}
if {[info exists env(RTOS)]} then {set RTOS $env(RTOS)} else {set RTOS LynxOS}

#    load the common f2vb/d2vb source

loadsource x2vbdiag

if {$RTOS == "VxWorks"} then {
    set X2VB_registerbase 0xf0e00000
    set X2VB_listrambase  0xf0e00800
} else {
    set X2VB_registerbase 0x00e00000
    set X2VB_listrambase  0x00e00800
}


proc define-x2vb-registers {} {
   global RTOS X2VB_registerbase X2VB_listrambase 
   global resource mname

   set register /$resource/$mname.D*
   set-footer "deleting register $register"
   EG "register :pattern '$register' !delete"

   define-register xVME DCSR "-a $X2VB_registerbase -o 0  -w 24 -d 16 -z x -p rw"
   define-register xVME DCCR "-a $X2VB_registerbase -o 2  -w 24 -d 16 -z x -p rw"
   define-register xVME DICR "-a $X2VB_registerbase -o 6  -w 24 -d 16 -z x -p rw"
   define-register xVME DIVR "-a $X2VB_registerbase -o 8  -w 24 -d 16 -z x -p rw"
   define-register xVME DLRAR "-a $X2VB_registerbase -o 10  -w 24 -d 16 -z x -p ro"
   define-register xVME DDRAR "-a $X2VB_registerbase -o 12  -w 24 -d 16 -z x -p rw"
   define-register xVME DCBRA "-a $X2VB_registerbase -o 16  -w 24 -d 32 -z x -p ro"
   define-register xVME DCBWA "-a $X2VB_registerbase -o 20  -w 24 -d 32 -z x -p ro"
   define-register bVME Dbram "-a [expr [read-register DDRAR] << 16] -w 32 -d 32 -z x -p rw -l 64"
if {$RTOS == "VxWorks"} then {
   define-register bVME Dlram "-a $X2VB_listrambase -w 16 -z x -p rw -l 96"
} else {
   define-register bVME Dlram "-a $X2VB_listrambase -w 24 -d 16 -z x -p rw -l 96"
}
}

proc define-register {class name attributes} {
   global resource mname

   set register /$resource/$mname.$name
   set-footer "defining register $register"
   EG "register :name '$register' :class '$class' !new"
   EG "register :name '$register' :object '$attributes'"
   clear-footer
}

proc add-format-option {} {
     global dumpopt
     :choice-stack FORMATOPT :label "format option" :strings [list raw "no ADC type" "format as LeCroy" "format as Silena"] :value $dumpopt
}


#    procedure adapted from Euroball EC task

proc format_the_dump {} {
  global dumpdata
  global dumpopt dumpstart dumpwords

#    format f2vb data RAM data held in global variable dumpdata

   set words [expr $dumpwords * 2]

   set-footer "formatting dump"

   set frame "dump"
   set win [self frame-name].$frame
   :frame $win :label  "Formatted Data Dump"  :transient T
   :layout T

   switch $dumpopt {
   0  {
        :galley G :rows 10 :cols 100 :deaf T :font F :strings ""
        :modify G :strings $dumpdata
      }

   1  {
        :galley G :rows 25 :cols 100 :deaf T :font F :strings ""
        set dump ""
        for {set i 0} {$i < $words} {incr i} {
           set word [lindex $dumpdata $i]

           set flag [expr $word >> 15 & 1]
           if {$flag == 1} then {
#    header
             :modify G :strings $dump
             :modify G :strings "\n\n"

             set dump "$word "
           } else {
             append dump "$word "
           }
        }
        :modify G :strings $dump
     }
   3 -
   2  {
        :galley G :rows 50 :cols 75 :deaf T :strings "" :font fixed
        :show T
#        :modify G :strings "dump starting at word offset $dumpstart for $words words\n"
           set vsn 0
        for {set i 0} {$i < $dumpwords} {incr i} {
           set word [lindex $dumpdata $i]

           if {$word == 0xffff} then {
               :modify G :strings "-------------------------------------------------------------------------\n"
               continue
           }
           if {$word == 0x8000} then {continue}

           set flag [expr $word >> 15 & 1]
           set Flag $flag

           if {[expr $word >> 12 & 15] == 0} then {

#    added to handle Silenas which do not put out header word flag bit

               if {[expr [lindex $dumpdata [expr $i+1]] >> 8 & 255] == 0} then {

#    this is a pattern word and so previous must be header 

                   set flag 1
               }
           }

           if {$flag == 1} then {
              :modify G :strings "\n"
              set vsn  [expr $word & 0xff]
              set sa [expr $word >> 8 & 0x7f]
              set data  [expr $word & 0xff]
              if {$dumpopt == 2} then {
#    LeCroy
                 set wc [expr $word >> 11 & 15]
                 if {$wc == 0} then {set wc 16}
                 set title "LeCroy Header Word - VSN= $vsn; WC= $wc"
              } else {
#    Silena 
                 set wc [expr $word >> 8 & 15]
                 incr i
                 set pw [lindex $dumpdata $i]
                 set title "Silena Header Word - F= $Flag; VSN= $vsn; VDC= $wc; PW= $pw"
              }
           } else {
              if {$dumpopt == 2} then {
#    LeCroy
                 set sa [expr $word >> 11 & 15]
                 set data  [expr $word & 0x7ff]
                 set title "Data Word - SubAddress= $sa; data= [format "%4x" $data]"
              } else {
#    Silena
                 set sa [expr $word >> 12 & 7]
                 set data  [expr $word & 0xfff]
                 set title "Data Word - SubAddress= $sa; data= [format "%4x" $data]"
              }
           }
           :modify G :strings "[format "   %1d %3d %6x   %40s" $Flag $sa $data $title] \n"

        }
     }

   }
   :modify G :see 0
   :layout B; :set fill none; :set anchor center
   :button B  :label "Dismiss" :command "destroy $win"
   :show T
   clear-footer
}





proc format_a_block {} {
  global dumpdata dumpwords

   set minvsn 1
   set maxvsn 20
   set minadc 0
   set maxadc 7
 
#    run the sort spy test program to capture a block

   set-footer "Obtaining a data block from the sort-spy"
   set dumpdata [exec sort-spy-dump 1]
   set dumpwords [llength $dumpdata]

   set-footer "formatting data block"

   set frame "format"
   :frame [self frame-name].$frame :label  "Formatted Data Block"   :toplevel T

        :galley G1 :rows 50 :cols 75 :deaf T :strings "" :font fixed
        :show T

        set vsn 0
        set bc 0
        set badvsn 0
        set badadc 0
        set badpw 0
        set badvdc 0
        set adc# 0
        set vsn# 0

        for {set i 5} {$i < $dumpwords} {incr i} {
           set word [lindex $dumpdata $i]
           incr bc

           if {$word == 0xffff} then {
               :modify G1 :strings "-------------------------------------------------------------------------\n"
               if {[lindex $dumpdata [expr $i+1]] == 0xffff} then {
                   incr bc
                   :modify G1 :strings "-------------------------------------------------------------------------\n"
                   break
               }
               continue
           }
           if {$word == 0x8000} then {continue}

           set flag [expr $word >> 15 & 1]
           set Flag $flag

           if {[expr $word >> 12 & 15] == 0} then {

#    added to handle Silenas which do not put out header word flag bit

               if {[expr [lindex $dumpdata [expr $i+1]] >> 8 & 255] == 0} then {

#    this is a pattern word and so previous must be header 

                   set flag 1
               }
           }

           if {$flag == 1} then {
                 :modify G1 :strings "\n"
                 incr vsn#

                 set sa [expr $word >> 8 & 0x7f]
                 if {$sa < 1 || $sa > 8} {incr badvdc}

                 set data  [expr $word & 0xff]

                 set vsn  [expr $word & 0xff]
                 if {$vsn < $minvsn || $vsn > $maxvsn} {incr badvsn}
                 set wc [expr $word >> 8 & 15]
                 incr i
                 incr bc
                 set pw [lindex $dumpdata $i]
                 if {$pw < 1 || $pw > 255} {incr badpw}
                 set title "Silena Header Word - F= $Flag; VSN= $vsn; VDC= $wc; PW= $pw"
           } else {
                 incr adc#
                 set sa [expr $word >> 12 & 15]
                 if {$sa < $minadc || $sa > $maxadc} {incr badadc}
                 set data  [expr $word & 0xfff]
                 set title "Data Word - SubAddress= $sa; data= [format "%4x" $data]"
           }
          
           :modify G1 :strings "[format "   %1d %3d %6x   %40s" $Flag $sa $data $title] \n"

        }

   :modify G1 :see 0

   set frame "summary"
   :frame [self frame-name].$frame :label  "Data Block Summary"   :toplevel T

   :galley G2 :rows 10 :cols 75 :deaf T :strings "" :font fixed
   :show T

   :modify G2 :strings "End of data after $bc data items\n"
   :modify G2 :strings "total adc header words [set vsn#]; total adc data words [set adc#]\n"
   :modify G2 :strings "bad VSNs $badvsn\n"
   :modify G2 :strings "bad SAs $badadc\n"
   :modify G2 :strings "bad PWs $badpw\n"
   :modify G2 :strings "bad VDCs $badvdc\n"
   :modify G2 :see 0

   clear-footer
}


