#    Diagnostic task for testing VME modules

set frame VMEDIAG
set title "VME Diagnostics"
set icon VMEDiag

set started 0
set Testing 0

#    by default rdw and rdl procs write to the log window
set LogFlag 1

set RoleList [list VME vme VXI vxi]

set DiagFileBase [file join $env(MIDASBASE) vme tcl diag-scripts]

set server "xxxx"
set VMECrateList {}

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set inputprompt  ""
set outputprompt ""
set insertprompt ">>"

set histlim 40
for {set i 0} {$i <= $histlim} {incr i} {lappend history ""}
set nextcommand $histlim

set A16Base 0x1000
set A24Base 0x010000
set A32Base 0x01000000

set AccessMenu [list A16 A24 A32]
set AccessMode A32

set mname Diag

set AbortFlag 0

proc click arg {
   global history
   global A16Base A24Base A32Base AccessMenu AccessMode LogFlag AbortFlag

   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   
   if {$c==":quit"} then {return [click-quit $f]}
   clear-footer
   switch -glob $w  {
   
     A16BASE    {set A16Base $v}
     A24BASE    {set A24Base $v}
     A32BASE    {set A32Base $v}
     
     ACCESS     {set AccessMode [lindex $AccessMenu $v]}
     LOGGING    {set LogFlag $v}
     ABORT      {set AbortFlag 1}
     
     SERVERT    {click-server $v}
     SERVERC    {click-server $i}
     
      CMD        {invoke "$v"}
      HISTORY    {
                 set i [first $v]
                 invoke "[lindex $history $i]"
                 }

      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

#    inital application start
proc start args {
   global started VMECrateList

#    built the choice menu for the server
   make-serverlist

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   
   click-server [lindex $VMECrateList 0]
}

proc click-trace {v} {
#    write-register TraceMode $v
#    :modify TRACE :value [read-register TraceMode]
}

proc click-server {v} {
    global server VMECrateList

    set server $v
    :modify SERVERT :value $server
    :modify SERVERC :value [lsearch $VMECrateList $server]

    set-footer "Connecting to server $server"
    define-SERVER-registers

    clear-footer
    log "Connected to server $server"

}


proc make-frame {} {
   global env VMECrateList
   global A16Base A24Base A32Base AccessMenu AccessMode
   global AbortFlag LogFlag

  set rows 8
  set cols 50

  :set autoreturn "F"

  :frame [self frame-name] :label "[self frame-title]" :show-footer T  \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize T

   :panel P1 :layout T :padx 10 :pady 5
     :layout H; :col-gap 5; :row-gap 10
       :text SERVERT         :label "VME Server"  :w 8
       :choice-stack SERVERC :label "" :menu $VMECrateList
       :hskip 50
       :checkbox TRACE :label "VME crate TraceMode" 
     :next-row T
     :layout V
       :text A16BASE  :label "A16 base address"  :w 12  :valuex 24c :value $A16Base
       :text A24BASE  :label "A24 base address"  :w 12  :valuex 24c :value $A24Base
       :text A32BASE  :label "A32 base address"  :w 12  :valuex 24c :value $A32Base
       :choice ACCESS :label "VME access mode" :strings $AccessMenu :value [lsearch $AccessMenu $AccessMode]
      :layout H
       :col-gap 10
       :button ABORT :label "Abort Command"
       :checkbox LOGGING :label "Enable Log" :value $LogFlag
       
   :panel P3 :layout T :padx 10
     :galley LOG :rows $rows :cols $cols :font variable
     configuretags

   :panel P4 :layout T :padx 10 :pady 10
     :layout H; :col-gap 50; :row-gap 10
     :button BAT :label "Execute file"
     :menubutton HISTORY   :label "History"  :strings [list ""]
     :button EMPTYLOG :label "Empty program log window" :command ":modify LOG :empty T"
     :next-row T
     :colour local entry foreground black
     :text CMD  :label "command:" :w [expr $cols - 10]
    bind [:path CMD].entry <Meta-k>         "restorecommand last"
    bind [:path CMD].entry <Meta-BackSpace> "zapcommand"
    bind [:path CMD].entry <Alt-KeyPress-k>         "restorecommand last"
    bind [:path CMD].entry <Alt-KeyPress-BackSpace> "zapcommand"
    bind [:path CMD].entry <Up> "restorecommand previous"
    bind [:path CMD].entry <Down> "restorecommand next"

     open-frame

     :galley_size LOG $rows $cols

    focus [:path CMD].entry
}


proc restorecommand {c} {
  global history histlim nextcommand 
  switch $c {
    last  {:modify CMD :value [lindex $history end]}
    previous  {
     :modify CMD :value [lindex $history $nextcommand]
     incr nextcommand -1
     if {$nextcommand < 0} then {set nextcommand 0}
    }
    next  {
     incr nextcommand 1
     if {$nextcommand > $histlim} then {set nextcommand $histlim}
     :modify CMD :value [lindex $history $nextcommand]
    }
    default {log "restorecommand $c?"}
  }
}

proc zapcommand {} {:modify CMD :value ""}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc click-bat args {
    global env server DiagFileBase

#    Here use the built in browser function
#    This uses the canvas function and requires a pale coloured background

    option add *Canvas.background  #c0c020
    set types {
      {{Command Files}      {.bat}   }
      {{All Files}   *        }
    }
    set batfile [tk_getOpenFile -parent [self frame-name] -title "Diagnostic script Selection"  \
       -initialdir $DiagFileBase -filetypes $types]
    option add *Canvas.background  $env(tkcanvasbackground)
    if {$batfile != ""} then {
       set directory [file dirname $batfile]
       set file [file tail $batfile]
       set z [midas-confirmation "Execute file $file for the VME card in VME crate $server"]
       if {$z == 0} then {return}
       uplevel #0 source $batfile
       set DiagFileBase $directory
    }
}


proc invoke {text} {
    global outputprompt inputprompt
    global history histmenu histlim nextcommand
    global AbortFlag LogFlag

    set AbortFlag 0

    set cmd ""
    foreach item [split $text ", "] {
        if {$item != ""} then {append cmd $item " "}
    }

    :modify LOG :usingtag taginput :strings "[set inputprompt]$cmd\n"

  if [info complete $cmd] {
       if {$cmd == "!!"} {
           set cmd [lindex $history end]
       } else {
           lappend history "$cmd"
           set history [lreplace $history 0 0]
       }
  
    if {[first $cmd] == "repeat"} then {
        set count [second $cmd]
        set l [string length "repeat $count"]
        set cmd [string range $cmd [expr $l+1] end]
    } else {set count 1}

    if {$count == 0} then {set count 100000}
    for {set i 1} {$i <= $count} {incr i} {
       update
       if {$AbortFlag == 1} {:modify LOG :strings "Abort\n"; break}
       set result [catch {eval $cmd} msg]
       if {$result != 0} {
            :modify LOG :usingtag tagoutput :strings "[set outputprompt]Error: $msg\n"
            if {$i <= $count} then {:modify LOG :usingtag tagoutput :strings "loop aborted due to error\n"}
            break;
       } else {
           if {$msg != ""} then {if {$LogFlag == 1 || $i == 1} then {:modify LOG :usingtag tagoutput :strings "$outputprompt$msg\n"}}
       }
    }
  }

    :modify CMD :value ""
    set nextcommand $histlim

    set histmenu {}
    set i 0
    foreach s $history {
       if {$s != ""} then {lappend histmenu "$i [string range "$s" 0 40]"}
       incr i
    }
    :modify HISTORY :strings $histmenu
}

proc log {s} {
  global insertprompt
  :modify LOG :usingtag taginsert :strings "$insertprompt$s\n"
}

#    register access procedures

proc define-SERVER-registers {} {
   global server EGreport

   set EGreport -1
      
      delete-registers "X.a??d\[?,??\]"

      define-register xVME X.a32d8  "-a0 -o0x0 -prw -w32 -d8 -fx"
      define-register xVME X.a32d16 "-a0 -o0x0 -prw -w32 -d16 -fx"
      define-register xVME X.a32d32 "-a0 -o0x0 -prw -w32 -d32 -fx"
      define-register xVME X.a24d8  "-a0 -o0x0 -prw -w24 -d8 -fx"
      define-register xVME X.a24d16 "-a0 -o0x0 -prw -w24 -d16 -fx"
      define-register xVME X.a24d32 "-a0 -o0x0 -prw -w24 -d32 -fx"
      define-register xVME X.a16d8  "-a0 -o0x0 -prw -w16 -d8 -fx -m0x29"
      define-register xVME X.a16d16 "-a0 -o0x0 -prw -w16 -d16 -fx"

#      define-register bVME B.a32d16 "-a0 -o0x0 -prw -w32 -d16 -fx"
#      define-register bVME B.a32d32 "-a0 -o0x0 -prw -w32 -d32 -fx"
#      define-register bVME B.a24d16 "-a0 -o0x0 -prw -w24 -d16 -fx"
#      define-register bVME B.a24d32 "-a0 -o0x0 -prw -w24 -d32 -fx"
#      define-register bVME B.a16d16 "-a0 -o0x0 -prw -w16 -d16 -fx"


#   delete-registers TraceMode
#   define-register generic TraceMode "-n TraceMode -t int"

   set EGreport 0
   
   midas-warning " VME crate access created - If the VME crate is rebooted you will have to \
       reselect the VME Crate Server"
}

proc define-register {class name attributes} {
   global server mname

   set register /$server/$mname.$name
   set-footer "defining register $register"
   EG "register :name '$register' :class '$class' !new"
   EG "register :name '$register' :object '$attributes'"
   clear-footer
}

proc delete-registers {name} {
   global server mname

   set register /$server/$mname.$name
   set-footer "deleting register $register"
   EG "register :pattern '$register' !delete"
}

proc modify-register {name attributes} {
   global server mname

   set register /$server/$mname.$name
   EG "register :name '$register' :object '$attributes'"
}

proc read-register {name} {
   global server mname
   set register /$server/$mname.$name
   return [EG "register :name '$register' ?value"]
}

proc write-register {name value} {
   global server mname
   set register /$server/$mname.$name
   return [EG "register :name '$register' :value '$value'"]
}

#    proc to read 32 bits from module
#    address is VME address relative to module base
proc rdl {address} {return [rd l $address]}

#    proc to read 16 bits from module
#    address is VME address relative to module base
proc rdw {address} {return [rd w $address]}

#    proc to read 8 bits from module
#    address is VME address relative to module base
proc rdb {address} {return [rd b $address]}

proc rd {width address} {
   global LogFlag EGallow EGreport
   global A16Base A24Base A32Base AccessMode

   switch $width {
       b   {set DataMode d8}
       w   {set DataMode d16}
       l   {set DataMode d32}
   }

   switch $AccessMode {
       A16  {set attributes "-a $A16Base -o [format "0x%04x" [expr $address]]"}
       A24  {set attributes "-a $A24Base -o [format "0x%04x" [expr $address]]"}
       A32  {set attributes "-a $A32Base -o [format "0x%04x" [expr $address]]"}
   }

   set EGallow 0x2000a
   set EGreport_SAVE $EGreport
   set EGreport -1

   set register X.[string tolower $AccessMode]$DataMode
   modify-register $register $attributes
   set r [read-register $register]

   set EGreport $EGreport_SAVE
   set EGallow 0xffffffff

   if {$r == 0x2000a} then {set r "BERR"}
   if {$LogFlag == 1} then {log "$AccessMode[string toupper $DataMode]: rd$width [format "0x%04x" [expr $address]]: $r"}
   return $r
}

proc rdblk {width length address} {
   global LogFlag
   global A16Base A24Base A32Base AccessMode

   switch $width {
       b   {set DataMode d8}
       w   {set DataMode d16}
       l   {set DataMode d32}
   }

   switch $AccessMode {
       A16  {set attributes "-a $A16Base -o [format "0x%04x" [expr $address]] -l $length"}
       A24  {set attributes "-a $A24Base -o [format "0x%04x" [expr $address]] -l $length"}
       A32  {set attributes "-a $A32Base -o [format "0x%04x" [expr $address]] -l $length"}
   }

   set register B.[string tolower $AccessMode]$DataMode
   modify-register $register $attributes

   set r [read-register $register]
   if {$LogFlag == 1} then {log "$AccessMode[string toupper $DataMode]: rd$width [format "0x%04x" [expr $address]]: $r"}
   return $r
}

#    proc to write 32 bits to module
#    address is VME address relative to module base
proc wrl {address args} {
   switch [llength $args] {
       0  {set r [wr l $address 0 0]}
       1  {set r [wr l $address [first $args] 0]}
       2  {set r [wr l $address [first $args] [second $args]]}
       default {midas-warning "wrong number of arguments in call to wrl -  $args"; return}
   }
}

#    proc to write 16 bits to module
#    address is VME address relative to module base
proc wrw {address args} {
   switch [llength $args] {
       0  {set r [wr w $address 0 0]}
       1  {set r [wr w $address [first $args] 0]}
       2  {set r [wr w $address [first $args] [second $args]]}
       default {midas-warning "wrong number of arguments in call to wrw -  $args"; return}
   }
}

#    proc to write 8 bits to module
#    address is VME address relative to module base
proc wrb {address args} {
   switch [llength $args] {
       0  {set r [wr b $address 0 0]}
       1  {set r [wr b $address [first $args] 0]}
       2  {set r [wr b $address [first $args] [second $args]]}
       default {midas-warning "wrong number of arguments in call to wrw -  $args"; return}
   }
}

proc wr {width address data flag} {
   global LogFlag EGallow EGreport
   global A16Base A24Base A32Base AccessMode

   switch $width {
       b   {set DataMode d8}
       w   {set DataMode d16}
       l   {set DataMode d32}
   }

   switch $AccessMode {
       A16  {set attributes "-a $A16Base -o [format "0x%04x" [expr $address]]"}
       A24  {set attributes "-a $A24Base -o [format "0x%04x" [expr $address]]"}
       A32  {set attributes "-a $A32Base -o [format "0x%04x" [expr $address]]"}
   }

   set EGallow 0x2000a
   set EGreport_SAVE $EGreport
   set EGreport -1

   set register X.[string tolower $AccessMode]$DataMode
   modify-register $register $attributes

   if {$LogFlag == 1} then {log "$AccessMode[string toupper $DataMode]: wr$width [format "0x%04x" [expr $address]]: [expr $data]"}

   set r [write-register $register [expr $data]]

   if {$r == 0x2000a} then {if {$LogFlag == 1} then {log "BERR"}}
   if {$flag == 0} then {
       set EGallow 0x2000a
       set r [read-register $register]
       set EGallow 0xffffffff
       if {$r == 0x2000a} then {set r "BERR"}
   } else {set r $data}

   set EGreport $EGreport_SAVE
   set EGallow 0xffffffff

   return $r
}

#    procedures taken from the vmecontrol task to obtain list of
#    available VME servers from the experiment file


proc make-serverlist {} {
  global VMECrateList RoleList

#    find VME crates available

   set VMECrateList ""
   if {[catch {eg "(experiment ?resources)"} m]} then {
       midas-warning "no experiment connected!!!"
   } else {
       foreach role $RoleList {
          set c [make-resource-list $m $role]
          if {$c != ""} then {append VMECrateList " $c"}
           
       }
   }
#    sort lists into sensible order
   set cratelist [lsort $VMECrateList]
   if {[llength $cratelist] == 0} then {
      midas-warning "no VME crates found in current experiment!!!"
      set VMECrateList {vme}
   } else {
      set VMECrateList {}
      foreach crate $cratelist {lappend VMECrateList $crate}
   }
}

proc make-resource-list {m role} {
   set l ""
   foreach item $m {
      set r [find-resource $item $role]
      if {$r != ""} then {append l " $r"}
   }
   return $l
}

proc find-resource {item r} {
   set l {}
   set role {}; set name {}; set type {}
   foreach p $item {
      switch [first $p] {
         name {set name [second $p]}
         role {set role [second $p]}
         type {set type [second $p]}
         default {}
      }
   }
   if {$type == "register" && $r == $role} then {set l $name}
   return $l
}

proc taskDelay {t} {after $t}

if {$Testing == 1} then {
proc EG {args} {log "EG $args"; return 1}
}
